/*
* Copyright © 2018 - 2019 Virtru Corporation
*
* SPDX - License - Identifier: MIT
*
*/
//
//  Virtru TDF3 SDK
//
//  Created by Sujan Reddy on 2019/07/18.
//

#ifndef VIRTRU_TDF3_SDK_VIRTRU_ENCRYPT_FILE_PARAM_H
#define VIRTRU_TDF3_SDK_VIRTRU_ENCRYPT_FILE_PARAM_H

#include <string>
#include <virtru_policy.h>

namespace virtru {

    /// Forward declaration
    class Client;

    class EncryptFileParam {
    public:
        /// Constructor
        /// NOTE: The out file path will have .html/.tdf extension appended to
        /// inFilePath based on protocol.
        /// \param inFilepath - The file on which the encryption is performed
        EncryptFileParam(const std::string& inFilePath);

        /// Constructor
        /// \param inFilepath - The file on which the encryption is performed
        /// \param outFilepath - The file path of the tdf after successful encryption
        EncryptFileParam(const std::string& inFilePath, const std::string& outFilePath);

        /// Default constructor not supported
        EncryptFileParam() = delete;

        /// Destructor
        ~EncryptFileParam();

        /// Copy constructor
        EncryptFileParam(const EncryptFileParam& encryptedFileParams);

        /// Assignment operator
        EncryptFileParam& operator=(const EncryptFileParam& encryptedFileParams);

        /// Move copy constructor
        EncryptFileParam(EncryptFileParam&& encryptedFileParams);

        /// Move assignment operator
        EncryptFileParam& operator=(EncryptFileParam&& encryptedFileParams);

    public:
        /// Set the mime type (format) of the file that will be encrypted
        /// NOTE: The default will be 'application/octet-stream' if not specified
        /// \param mimeType - The mime type of the file
        void setMimeType(const std::string& mimeType);

        /// Set the policy that to be applied to the tdf
        /// NOTE: If there is no policy defined for the tdf, default policy settings
        /// will be applied
        /// \param policy - The policy object
        void setPolicy(const Policy& policy);

        /// Set the display name for the tdf, this name shows up on the Virtru dashboard
        /// NOTE: If not set, the basename of inFilePath will be used
        /// \param displayName - The display name for the tdf
        void setDisplayName(const std::string& displayName);

        /// Set the display message for the tdf, this message shows up on the Virtru dashboard
        /// \param displayMessage - The display message for the tdf
        void setDisplayMessage(const std::string& displayMessage);

    protected:
        friend Client;

        std::string m_inputFilePath;
        std::string m_outFilePath;
        std::string m_mimeType;
        std::string m_displayName;
        std::string m_displayMessage;
        Policy m_policy;
    };
}




#endif //VIRTRU_TDF3_SDK_VIRTRU_ENCRYPT_FILE_PARAM_H
