/*
* Copyright © 2018 - 2019 Virtru Corporation
*
* SPDX - License - Identifier: MIT
*
*/
//
//  Virtru TDF3 SDK
//
//  Created by Pat Mancuso on 2019/05/06

#include <fstream>
#include <iostream>

// Only interface required.
#include <virtru_client.h>

using namespace virtru;

// User credentials
const std::string appId = "11111111-1111-1111-1111-111111111111";  // Put owner appId here
const std::string user = "your-user-name@your-domain.com";         // Put owner email here

// Sample file names and data
const char* samplePlaintextFilename = "samplePlaintext.txt";
const char* sampleCiphertextFilename = "sampleCiphertext.txt.html";
const char* sampleDecryptedFilename = "sampleDecrypted.txt";

const char* testTextData = "This is a simple test.";

// Function to create a data file to operate on
void createSampleData()
{
    // Housekeeping - clean start
    std::remove(samplePlaintextFilename);
    std::remove(sampleCiphertextFilename);
    std::remove(sampleDecryptedFilename);

    // Create a file to work on
    // This does not specify a path, and therefore operates on files in the current working directory
    std::ofstream samplePlaintextStream = std::ofstream(samplePlaintextFilename, std::ofstream::out);
    if (samplePlaintextStream.fail()) {
        throw std::runtime_error("Could not create input file");
    }
    else
    {
        // Write data to file and close it
        samplePlaintextStream << testTextData;
        samplePlaintextStream.close();
    }
}

// Simple application to encrypt and decrypt files using the Virtru SDK for C++
int main()
{
    try {
        createSampleData();

        // Create an instance of the Virtru client.
        Client client {user, appId};

        // Use the client instance to encrypt a file
        auto policyId = client.encryptFile({samplePlaintextFilename, sampleCiphertextFilename});

        // Use the client instance to decrypt the file again
        client.decryptFile(sampleCiphertextFilename, sampleDecryptedFilename);

        std::cout << "Encrypt and decrypt operations completed successfully" << std::endl;

        // Add a user
        Policy policy;
        policy.shareWithUsers({"someOtherUser@place.com"});

        // Set an expiration date in the future
        policy.addExpiration("2099-05-24");

        // Update the policy changes at the server
        client.updatePolicyForUUID(policy, policyId);

        std::cout << "Policy update operations completed successfully" << std::endl;
    }
    catch (const std::exception& exception) {
        std::cerr << "Exception " << exception.what() << std::endl;
        return -1;
    }
    return 0;
}
